package com.jkcredit.illegal.info.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.jkcredit.illegal.info.constant.CommonConstant;
import com.jkcredit.illegal.info.model.*;
import com.jkcredit.illegal.info.service.CriminalInfoService;
import com.jkcredit.illegal.info.service.IllegalInfoService;
import com.jkcredit.illegal.info.util.AesUtil;
import com.jkcredit.illegal.info.util.OkHttpUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Service;

/**
 * @description:
 * @author: xusonglin
 * @create: 2020/9/22 11:51
 * @version: V1.0
 **/
@Slf4j
@Service
public class IllegalInfoServiceImpl implements IllegalInfoService {
    @Value("${illegalInfo.url}")
    private String illegalInfoUrl;
    @Value("${illegalInfo.vehicleUrl}")
    private String vehicleIllegalInfoUrl;
    @Value("${illegalInfo.decodeKey}")
    private String decodeKey;
    @Autowired
    StringRedisTemplate stringRedisTemplate;
    @Autowired
    CriminalInfoService criminalInfoService;

    private static final Log CHARGE_LOGGER = LogFactory.getLog("CHARGE_LOGGER");

    /**
     * 社会风险预警-超时返回一致
     * @param requestParams 姓名 身份证号 流水号
     * @return 结果
     */
    @Override
    public IllegalInfoResult checkIllegalInfo(IllegalInfoRequestParam requestParams) {
        String body = JSON.toJSONString(requestParams);

        Long startTime = System.currentTimeMillis();
        String result = OkHttpUtil.doPost(illegalInfoUrl, body, CommonConstant.PERSON_TIME_OUT);
        log.info("调用上游接口-流水号:{}, 入参:{}, 返回:{}, 时延:{}", requestParams.getTraceId(), body, result, System.currentTimeMillis() - startTime);
        if (StringUtils.isBlank(result)) {
            log.info("checkIllegalInfoRequestError-流水号:{}, 入参:{}, 请求上游失败无返回", requestParams.getTraceId(), body);
            stringRedisTemplate.boundValueOps(CommonConstant.PERSON_ILLEGAL_INFO_TIME_OUT).increment();
        }

        IllegalInfoResult illegalInfoResult;
        try {
            if (StringUtils.isBlank(result)) {
                illegalInfoResult = new IllegalInfoResult().normalResult();
                CHARGE_LOGGER.info(JSON.toJSONString(new IllegalInfoChargeObject(illegalInfoResult, false, requestParams.getTraceId())));
                return illegalInfoResult;
            } else {
                illegalInfoResult = JSON.toJavaObject(JSON.parseObject(result), IllegalInfoResult.class);
            }
        } catch (Exception e) {
            log.error("checkIllegalInfo解析异常-流水号:{}, 入参:{}, 解析上游返回异常:{}", requestParams.getTraceId(), body, e);
            illegalInfoResult = new IllegalInfoResult().normalResult();
            CHARGE_LOGGER.info(JSON.toJSONString(new IllegalInfoChargeObject(illegalInfoResult, false, requestParams.getTraceId())));
            return illegalInfoResult;
        }

        // 返回4个参数，code0表示结果正常，其他表示异常
        if (!illegalInfoResult.getCode().equals("0")
                || illegalInfoResult.getIsDrugs().equals("2")
                || illegalInfoResult.getIsEscape().equals("2")
                || illegalInfoResult.getIsPedigree().equals("2")) {
            // 查询错误，不计费
            CHARGE_LOGGER.info(JSON.toJSONString(new IllegalInfoChargeObject(illegalInfoResult, false, requestParams.getTraceId())));
            stringRedisTemplate.boundValueOps(CommonConstant.PERSON_ILLEGAL_INFO_QUERY_FAILED).increment();
            log.info("checkIllegalInfoResultError-流水号:{}, 上游结果异常", requestParams.getTraceId());
            illegalInfoResult = new IllegalInfoResult().normalResult();
        } else {
            // 查询成功，计费
            CHARGE_LOGGER.info(JSON.toJSONString(new IllegalInfoChargeObject(illegalInfoResult, true, requestParams.getTraceId())));
        }
        return illegalInfoResult;
    }

    @Override
    public VehicleIllegalInfoResult checkVehicleIllegalInfo(String params) {
        try {
            params = AesUtil.decryAES(decodeKey, params);
        } catch (Exception e) {
            log.info("解析参数失败, 入参:{}", params);
            return null;
        }
        VehicleIllegalInfoRequestParam requestParam = getVehicleIllegalInfoRequestParam(params);
        String body = JSON.toJSONString(requestParam);

        Long startTime = System.currentTimeMillis();
        String result = OkHttpUtil.doPost(vehicleIllegalInfoUrl, body, CommonConstant.VEHICLE_TIME_OUT);
        log.info("调用上游接口-流水号:{}, 入参:{}, 返回:{}, 时延:{}", requestParam.getTraceId(), body, result, System.currentTimeMillis() - startTime);
        if (StringUtils.isBlank(result)) {
            log.info("checkVehicleIllegalInfoRequestError-流水号:{}, 入参:{}, 请求上游失败无返回", requestParam.getTraceId(), body);
            stringRedisTemplate.boundValueOps(CommonConstant.VEHICLE_ILLEGAL_INFO_TIME_OUT).increment();
            return null;
        }
        VehicleIllegalInfoResult vehicleIllegalInfoResult = JSON.toJavaObject(JSON.parseObject(result), VehicleIllegalInfoResult.class);
        // code0表示结果正常，其他表示异常
        if (!vehicleIllegalInfoResult.getCode().equals("0")
                || vehicleIllegalInfoResult.getResult().equals("2")) {
            // 查询错误，不计费
            CHARGE_LOGGER.info(JSON.toJSONString(new VehicleIllegalInfoChargeObject(vehicleIllegalInfoResult, false, requestParam.getTraceId())));
            stringRedisTemplate.boundValueOps(CommonConstant.VEHICLE_ILLEGAL_INFO_QUERY_FAILED).increment();
            log.info("checkVehicleIllegalInfoResultError-流水号:{}, 上游结果异常", requestParam.getTraceId());
        } else {
            // 查询成功，计费
            CHARGE_LOGGER.info(JSON.toJSONString(new VehicleIllegalInfoChargeObject(vehicleIllegalInfoResult, true, requestParam.getTraceId())));
        }
        return vehicleIllegalInfoResult;
    }

    private VehicleIllegalInfoRequestParam getVehicleIllegalInfoRequestParam(String params) {
        JSONObject jsonObject = JSON.parseObject(params);
        VehicleIllegalInfoRequestParam requestParam = new VehicleIllegalInfoRequestParam();
        requestParam.setName(jsonObject.getString("name"));
        requestParam.setPlateNumber(jsonObject.getString("plateNumber"));
        requestParam.setTraceId(jsonObject.getString("traceId"));
        requestParam.setIdCode(jsonObject.getString("idCode"));
        requestParam.setPlateColor(jsonObject.getString("plateColor"));
        return requestParam;
    }

    @Override
    public IllegalInfoResult checkPersonIllegalInfo(String params) {
        try {
            params = AesUtil.decryAES(decodeKey, params);
        } catch (Exception e) {
            log.info("解析参数失败, 入参:{}", params);
            return null;
        }
        IllegalInfoRequestParam requestParams = getIllegalInfoRequestParam(params);
        String body = JSON.toJSONString(requestParams);

        Long startTime = System.currentTimeMillis();
        String result = OkHttpUtil.doPost(illegalInfoUrl, body, CommonConstant.PERSON_TIME_OUT);
        log.info("调用上游接口-流水号:{}, 入参:{}, 返回:{}, 时延:{}", requestParams.getTraceId(), body, result, System.currentTimeMillis() - startTime);
        if (StringUtils.isBlank(result)) {
            log.info("checkIllegalInfoRequestError-流水号:{}, 入参:{}, 请求上游失败无返回", requestParams.getTraceId(), body);
            stringRedisTemplate.boundValueOps(CommonConstant.PERSON_ILLEGAL_INFO_TIME_OUT).increment();
        }

        IllegalInfoResult illegalInfoResult;
        try {
            if (StringUtils.isBlank(result)) {
                illegalInfoResult = new IllegalInfoResult().normalResult();
                CHARGE_LOGGER.info(JSON.toJSONString(new IllegalInfoChargeObject(illegalInfoResult, false, requestParams.getTraceId())));
                return illegalInfoResult;
            } else {
                illegalInfoResult = JSON.toJavaObject(JSON.parseObject(result), IllegalInfoResult.class);
            }
        } catch (Exception e) {
            log.error("checkIllegalInfo解析异常-流水号:{}, 入参:{}, 解析上游返回异常:{}", requestParams.getTraceId(), body, e);
            illegalInfoResult = new IllegalInfoResult().normalResult();
            CHARGE_LOGGER.info(JSON.toJSONString(new IllegalInfoChargeObject(illegalInfoResult, false, requestParams.getTraceId())));
            return illegalInfoResult;
        }

        // 返回4个参数，code0表示结果正常，其他表示异常
        if (!illegalInfoResult.getCode().equals("0")
                || illegalInfoResult.getIsDrugs().equals("2")
                || illegalInfoResult.getIsEscape().equals("2")
                || illegalInfoResult.getIsPedigree().equals("2")) {
            // 查询错误，不计费
            CHARGE_LOGGER.info(JSON.toJSONString(new IllegalInfoChargeObject(illegalInfoResult, false, requestParams.getTraceId())));
            stringRedisTemplate.boundValueOps(CommonConstant.PERSON_ILLEGAL_INFO_QUERY_FAILED).increment();
            log.info("checkIllegalInfoResultError-流水号:{}, 上游结果异常", requestParams.getTraceId());
        } else {
            // 查询成功，计费
            CHARGE_LOGGER.info(JSON.toJSONString(new IllegalInfoChargeObject(illegalInfoResult, true, requestParams.getTraceId())));
        }
        return illegalInfoResult;
    }

    private IllegalInfoRequestParam getIllegalInfoRequestParam(String params) {
        JSONObject jsonObject = JSON.parseObject(params);
        IllegalInfoRequestParam requestParam = new IllegalInfoRequestParam();
        requestParam.setName(jsonObject.getString("name"));
        requestParam.setTraceId(jsonObject.getString("traceId"));
        requestParam.setIdCode(jsonObject.getString("idCode"));
        return requestParam;
    }

    private IllegalInfoResult getCriminalInfoResult(IllegalInfoRequestParam requestParams) {
        CriminalInfoRequestObject requestObject = new CriminalInfoRequestObject();
        BeanUtils.copyProperties(requestParams, requestObject);
        CriminalInfoResult criminalInfoResult = criminalInfoService.getCriminalInfo(requestObject);
        IllegalInfoResult illegalInfoResult = new IllegalInfoResult().normalResult();
        if (criminalInfoResult == null) {
            return illegalInfoResult;
        } else {
            /**
             * 前科 8
             * 涉毒 14和26
             * 在逃 50
             * 未命中 2
             */
            if (criminalInfoResult.getScore().equals("14") || criminalInfoResult.getScore().equals("26")) {
                // 涉毒
                illegalInfoResult.setIsDrugs("1");
            } else if (criminalInfoResult.getScore().equals("50")) {
                // 在逃
                illegalInfoResult.setIsEscape("1");
            } else if (criminalInfoResult.getScore().equals("8")) {
                // 前科
                illegalInfoResult.setIsPedigree("1");
            } else {
                // 未命中重点人
                return illegalInfoResult;
            }
            return illegalInfoResult;
        }
    }

    /**
     * 社会风险预警-超时跳转接口
     * @param requestParams 姓名 身份证号 流水号
     * @return 结果
     */
    @Override
    public IllegalInfoResult checkIllegalInfoV2(IllegalInfoRequestParam requestParams) {
        String body = JSON.toJSONString(requestParams);
        Long startTime = System.currentTimeMillis();
        String result = OkHttpUtil.doPost(illegalInfoUrl, body, CommonConstant.PERSON_TIME_OUT);
        log.info("调用上游接口-流水号:{}, 入参:{}, 返回:{}, 时延:{}", requestParams.getTraceId(), body, result, System.currentTimeMillis() - startTime);
        if (StringUtils.isBlank(result)) {
            log.info("checkIllegalInfoRequestError-流水号:{}, 入参:{}, 请求上游失败无返回", requestParams.getTraceId(), body);
            stringRedisTemplate.boundValueOps(CommonConstant.PERSON_ILLEGAL_INFO_TIME_OUT).increment();
            IllegalInfoResult illegalInfoResult = getCriminalInfoResult(requestParams);
            CHARGE_LOGGER.info(JSON.toJSONString(new IllegalInfoChargeObject(illegalInfoResult, false, requestParams.getTraceId())));
            return illegalInfoResult;
        }

        IllegalInfoResult illegalInfoResult;
        try {
            illegalInfoResult = JSON.toJavaObject(JSON.parseObject(result), IllegalInfoResult.class);
        } catch (Exception e) {
            log.error("checkIllegalInfo解析异常-流水号:{}, 入参:{}, 解析上游返回异常:{}", requestParams.getTraceId(), body, e);
            illegalInfoResult = getCriminalInfoResult(requestParams);
            CHARGE_LOGGER.info(JSON.toJSONString(new IllegalInfoChargeObject(illegalInfoResult, false, requestParams.getTraceId())));
            return illegalInfoResult;
        }

        // 返回4个参数，code0表示结果正常，其他表示异常
        if (!illegalInfoResult.getCode().equals("0")
                || illegalInfoResult.getIsDrugs().equals("2")
                || illegalInfoResult.getIsEscape().equals("2")
                || illegalInfoResult.getIsPedigree().equals("2")) {
            // 查询错误，不计费
            CHARGE_LOGGER.info(JSON.toJSONString(new IllegalInfoChargeObject(illegalInfoResult, false, requestParams.getTraceId())));
            stringRedisTemplate.boundValueOps(CommonConstant.PERSON_ILLEGAL_INFO_QUERY_FAILED).increment();
            log.info("checkIllegalInfoResultError-流水号:{}, 上游结果异常", requestParams.getTraceId());
            illegalInfoResult = getCriminalInfoResult(requestParams);
        } else {
            // 查询成功，计费
            CHARGE_LOGGER.info(JSON.toJSONString(new IllegalInfoChargeObject(illegalInfoResult, true, requestParams.getTraceId())));
        }
        return illegalInfoResult;
    }

    /**
     * 社会风险预警
     * @param requestParams 姓名 身份证号 流水号
     * @return 结果
     */
    @Override
    public IllegalInfoResult checkIllegalInfoV3(IllegalInfoRequestParam requestParams) {
        String body = JSON.toJSONString(requestParams);

        Long startTime = System.currentTimeMillis();
        String result = OkHttpUtil.doPost(illegalInfoUrl, body, CommonConstant.PERSON_TIME_OUT);
        log.info("调用上游接口-流水号:{}, 入参:{}, 返回:{}, 时延:{}", requestParams.getTraceId(), body, result, System.currentTimeMillis() - startTime);
        if (StringUtils.isBlank(result)) {
            log.info("调用上游接口-流水号:{}, 入参:{}, 请求上游失败无返回", requestParams.getTraceId(), body);
            stringRedisTemplate.boundValueOps(CommonConstant.PERSON_ILLEGAL_INFO_TIME_OUT).increment();
            return null;
        }

        IllegalInfoResult illegalInfoResult = JSON.toJavaObject(JSON.parseObject(result), IllegalInfoResult.class);
        // 返回4个参数，code0表示结果正常，其他表示异常
        if (!illegalInfoResult.getCode().equals("0")
                || illegalInfoResult.getIsDrugs().equals("2")
                || illegalInfoResult.getIsEscape().equals("2")
                || illegalInfoResult.getIsPedigree().equals("2")) {
            // 查询错误，不计费
            CHARGE_LOGGER.info(JSON.toJSONString(new IllegalInfoChargeObject(illegalInfoResult, false, requestParams.getTraceId())));
            stringRedisTemplate.boundValueOps(CommonConstant.PERSON_ILLEGAL_INFO_QUERY_FAILED).increment();
        } else {
            // 查询成功，计费
            CHARGE_LOGGER.info(JSON.toJSONString(new IllegalInfoChargeObject(illegalInfoResult, true, requestParams.getTraceId())));
        }
        return illegalInfoResult;
    }
}
